
/**********************************************************************************
***********************************************************************************
***                                                                             ***
***     Programm zur Mitgliedererfassung mit RFID-Tags                          ***
***     und Erfassung der Daten auf einem SQL-Server                            ***
***     Version 0.540                                                           ***
***                                                                             ***
***                                                                             ***
***********************************************************************************
***********************************************************************************



Typisches layout Rfid rc522
 * --------------------------------------------------------------------------------
 *             MFRC522      Arduino       Arduino   Arduino        Variable
 *             Reader/PCD   Uno           Mega      Nano v3
 * Signal      Pin          Pin           Pin       Pin
 * --------------------------------------------------------------------------------
 * RST/Reset   RST          9             5         D9            RST_PIN
 * SPI SS CS   SDA(SS)      10            53        D10           SS_PIN_522   ---pin 53 für RC522
 * SPI MOSI    MOSI         11 / ICSP-4   51        D11           MOSI_PIN
 * SPI MISO    MISO         12 / ICSP-1   50        D12           MISO_PIN
 * SPI SCK     SCK          13 / ICSP-3   52        D13           SCK_PIN
 * SS PIN LS                              10    für logger       SS_PIN_LS




 * Logger-Shield
 *
 * SD card attached to SPI bus as follows:
 ** MOSI - pin 11
 ** MISO - pin 12
 ** CLK - pin 13
 ** CS - pin 4                                          --- pin 4 für Logger

Hinweise Umlaute für Standardtexte auf dem Display:
#define STR_auml "\xe1"
#define STR_Auml "Ae"
#define STR_uuml "\365"
#define STR_Uuml "Ue"
#define STR_ouml "\357"
#define STR_Ouml "Oe"
#define STR_szlig "\342"

Datenbank feuerwehr mit den Tabellen:
-------------
logging:
unixtime
Mitgliedsnummer	char(8)
Name char(25)
---------------------
mitglieder:
Mitgliedsnummer	char(8)
Name	char(25)
Rfid   char(8)
---------------------
tempanmeldung (Diese Tabelle wird automatisch erstellt und gelöscht):
Mitgliedsnummer varchar(8)
--------------------


*/

#include <SPI.h>
#include <MFRC522.h>
#include <LiquidCrystal_I2C.h>
#include <Wire.h>
#include <Ethernet.h>
#include <sha1.h>
#include <mysql.h>



// Setze I2C Adresse für Display mit 20 Zeichen, 4 Zeilen
LiquidCrystal_I2C lcd (0x27, 20, 4);

//const uint8_t chipSelect = 10;





int ok = 22;      //Pin für den Schaltereingang
int nein = 24;    //Pin für den Schaltereingang
int buzzer = 26;  //Pin für Buzzzer
int a = 0;
int systemmode = 0;

String Textlcdlast;


//für alle SPI-Geräte
#define RST_PIN		5		// angepasst für 2560
#define MOSI_PIN        11               // angepasst für 2560 
#define MISO_PIN        12              // angepasst für 2560
#define SCK_PIN         13               // angepasst für 2560 


//für den KartenLeser
#define SS_PIN_522	53              // angepasst für 2560

//für das Ethernet-Schield

#define SS_PIN_LS      10


byte readCard[4];		//

MFRC522 mfrc522(SS_PIN_522, RST_PIN);	// Create MFRC522 instance




byte Wappen1[8] = {
  B01111,
  B01100,
  B01110,
  B01111,
  B01111,
  B01111,
  B01111,
  B01111
};

byte Wappen2[8] = {
  B11111,
  B00000,
  B00000,
  B00000,
  B11000,
  B11110,
  B11110,
  B11100
};

byte Wappen3[8] = {
  B11110,
  B00010,
  B00010,
  B00110,
  B01110,
  B01110,
  B11110,
  B11110

};
byte Wappen4[8] = {
  B01000,
  B01011,
  B01111,
  B01100,
  B00111,
  B00111,
  B00001,
  B00000

};
byte Wappen5[8] = {
  B00001,
  B11000,
  B10001,
  B00101,
  B11101,
  B11001,
  B11011,
  B11111

};
byte Wappen6[8] = {
  B11110,
  B00010,
  B11010,
  B11110,
  B11100,
  B11100,
  B10000,
  B00000

};

String value_return;

//char* Name;
String User;
String Mitgliedsnummer;
String Cardteil0;
String Cardteil1;
String Cardteil2;
String Cardteil3;
String Date;

int rows = 0;

byte mac_addr[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
byte ip[] = { 192, 168, 2, 123 };

//IPAddress server_addr(192, 168, 2, 111);
//IPAddress server_addr(192, 168, 2, 112);
//IPAddress server_addr(192, 168, 2, 50);
IPAddress server_addr(192, 168, 2, 211);

char user[] = "arduino";
char password[] = "minex";

Connector my_conn;        // The Connector/Arduino reference




char SQL_0[90]; //Diese Variable wird mit der Kommunikation zum Server gefüllt und anschließend immer wieder geleert.
char SQL_1[] = "SELECT Name FROM ";
char SQL_11[] = "SELECT Mitgliedsnummer FROM ";

char SQL_2[] = "feuerwehr.mitglieder WHERE rfid= ";
char SQL_SCHON_ANGEMELDET[] = "SELECT Mitgliedsnummer FROM feuerwehr.tempanmeldung WHERE Mitgliedsnummer= ";
const char INSERT_TEXT_TEMP[] = "INSERT INTO feuerwehr.tempanmeldung VALUES(%s)";
char CREATE_TABLE[] = "CREATE TEMPORARY TABLE IF NOT EXISTS feuerwehr.tempanmeldung(Mitgliedsnummer varchar(8),PRIMARY KEY (Mitgliedsnummer))";

//CREATE TABLE IF NOT EXISTS
//char DROP_TABLE[] = "DROP TABLE IF EXISTS feuerwehr.tempanmeldung";
const char INSERT_TEXT_LOG[] = "INSERT INTO feuerwehr.logging (Mitgliedsnummer,Name) VALUES(%s,%s)";
const char DATUM[] = "SELECT DATE_FORMAT(NOW(), '%d.%m.%Y %T')";


//**********************************************************************************
//****                                                                          ****
//****                                 Setup                                    ****
//****                                                                          ****
//**********************************************************************************

void setup() {
  pinMode(SS_PIN_522, OUTPUT);
  pinMode(SS_PIN_LS, OUTPUT);
  pinMode(buzzer, OUTPUT);
  pinMode(ok, INPUT);
  pinMode(nein, INPUT);
  digitalWrite(SS_PIN_522, HIGH);
  digitalWrite(SS_PIN_LS, LOW);
  digitalWrite(buzzer, LOW);
  lcd.init();
  lcd.backlight();
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("startet...........");
  lcd.blink();

  Serial.begin(9600);

  SPI.begin();			// Initialisierung SPI bus
  Ethernet.begin(mac_addr, ip);
  /*if (Ethernet.begin(mac_addr) == 0)
  { lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Kein Netzwerk");
    while (true);
  }*/

  delay(1501);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("verbindet Server..");
  if (my_conn.mysql_connect(server_addr, 3306, user, password)) {
    delay(1501);
  }
  else
  { lcd.setCursor(0, 0);
    lcd.print("Kein Server....");
  }
  User = "leer";
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Eigene IP:");
  lcd.setCursor(0, 1);
  lcd.print(Ethernet.localIP());
  lcd.setCursor(0, 2);
  lcd.print("Server IP:");
  lcd.setCursor(0, 3);
  lcd.print(server_addr);
  delay(501);
  lcd.noBlink();
  //do_query(DROP_TABLE); //Löschen der alten temporären Tabelle.
  //do_query(CREATE_TABLE);//Anlegen einer neuen temporären Tabelle die überprüft, ob User schon angemeldet.

  digitalWrite(SS_PIN_LS, HIGH);
  mfrc522.PCD_Init();		// Init MFRC522
  if (digitalRead(nein) == HIGH) {
    systemmode = 1;
    Serial.println("sysmode test");
  }
  // Schreibe Begrüssungstext auf die LCD.
  if (systemmode == 0) {
    lcd.clear();
    lcd.createChar(0, Wappen1);
    lcd.createChar(1, Wappen2);
    lcd.createChar(2, Wappen3);
    lcd.createChar(3, Wappen4);
    lcd.createChar(4, Wappen5);
    lcd.createChar(5, Wappen6);
    lcd.setCursor(0, 0);
    lcd.print("Anwesenheitsliste");
    lcd.setCursor(0, 1);
    lcd.print("der JF Musterstadt");
    delay(2000);

    lcd_bitte_anmelden();
  }
  else {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Testmodus gestartet");
    delay(1000);
  }



}
//**********************************************************************************
//****                                                                          ****
//****                           Loop                                           ****
//****                                                                          ****
//**********************************************************************************
void loop() {
  a = 0;

  //**********************************************************************************
  //****                       Testmodus zum lesen der Kartendaten                ****
  //**********************************************************************************

  if (systemmode == 1) {
    if ( ! mfrc522.PICC_IsNewCardPresent()) { //If a new PICC placed to RFID reader continue
      return ;
    }
    if ( ! mfrc522.PICC_ReadCardSerial()) { //Since a PICC placed get Serial and continue
      return ;
    }

    Kartenlesen();

    String Textlcd = Cardteil0 + Cardteil1 + Cardteil2 + Cardteil3;


    if (Textlcd != Textlcdlast) {

      lcd.clear();
      lcd.setCursor(0, 1);
      lcd.print(Textlcd);
      Serial.println(Textlcd);
      Textlcdlast = Textlcd;
    }
    return;
  }
  //**********************************************************************************
  //****                       Ende Testmodus                                     ****
  //**********************************************************************************




  //Kartenleser einschalten

  digitalWrite(SS_PIN_522, LOW);
  pinMode(SS_PIN_522, OUTPUT);



  //  PICCs lesen
  if ( ! mfrc522.PICC_IsNewCardPresent()) { //If a new PICC placed to RFID reader continue
    return ;
  }
  if ( ! mfrc522.PICC_ReadCardSerial()) { //Since a PICC placed get Serial and continue
    return ;
  }

  Kartenlesen();

  String KText99 = "'" + Cardteil0 + Cardteil1 + Cardteil2 + Cardteil3 + "'";

  int KText99_len = KText99.length() + 1;
  char KText99_char[KText99_len];
  KText99.toCharArray(KText99_char, KText99_len);

  inBearbeitung();
//zunächst wird die SQl-Abfrage zusammengebaut
  strcat (SQL_0, SQL_1);
  strcat (SQL_0, SQL_2);
  strcat (SQL_0, KText99_char);
///..und hier krachts wenn der Server nicht mehr da ist
  do_query(SQL_0);
  SQL_0[0] = (char)0;
  User = value_return;




  strcat (SQL_0, SQL_11);
  strcat (SQL_0, SQL_2);
  strcat (SQL_0, KText99_char);

  do_query(SQL_0);
  SQL_0[0] = (char)0;

  Mitgliedsnummer = value_return;

  mfrc522.PICC_HaltA(); // Stop reading


  if (rows == 0)
  { karte_ungueltig();
    return;
  }


  Mitgliedsnummer = "'" + Mitgliedsnummer + "'";
  int Mitgliedsnummer_len = Mitgliedsnummer.length() + 1;
  char Mitgliedsnummer_c[Mitgliedsnummer_len + 1];
  Mitgliedsnummer.toCharArray(Mitgliedsnummer_c, Mitgliedsnummer_len);

  do_query(CREATE_TABLE);//Anlegen einer neuen temporären Tabelle die überprüft, ob User schon angemeldet.

  strcat (SQL_0, SQL_SCHON_ANGEMELDET);
  strcat (SQL_0, Mitgliedsnummer_c);
  do_query(SQL_0);

  if (rows > 0)
  { lcd_schon_angemeldet();
    delay (1000);
    lcd_bitte_anmelden();
    SQL_0[0] = (char)0;
    return;
  }

  SQL_0[0] = (char)0;

  //Zeit vom Server holen
  do_query(DATUM);
  Date = value_return;

  //*****************************************************************************************
  //Jetzt muss der Kartenleser ausgeschaltet und das Logger-Shield eingeschaltet werden.
  //*****************************************************************************************
  digitalWrite(SS_PIN_522, HIGH);
  digitalWrite(SS_PIN_LS, LOW);



  lcdgrundansicht();

  lcd.setCursor(0, 1);
  lcd.print(User);

  lcd.setCursor(10, 0);
  lcd.print(Mitgliedsnummer);
  lcd.setCursor(0, 2);
  // Zeitansicht
  lcd.print(Date);

  lcd.setCursor(6, 3);
  lcd.print(KText99);

  lcd.setCursor(16, 3);
  lcd.print("OK?");



  /////////////////Taster Überprüfung////////////////////////////////
  //Bestätigung ob die Daten geschrieben werden sollen
  digitalWrite(buzzer, HIGH);
  delay(100);
  digitalWrite(buzzer, LOW);
  while (a == 0) {
    if (digitalRead(ok) == HIGH) {
      a = 1;
      digitalWrite(buzzer, HIGH);
      delay(175);
      digitalWrite(buzzer, LOW);
      delay(175);
      digitalWrite(buzzer, HIGH);
      delay(175);
      digitalWrite(buzzer, LOW);
      lcd.clear();
      inBearbeitung();
    }
    if (digitalRead(nein) == HIGH) {

      lcd.clear();

      lcd.print("Abgebrochen");
      lcd.setCursor(0, 0);
      for (uint8_t i = 0; i < 20; i++)
      {
        digitalWrite(buzzer, HIGH);
        delay(30);

        digitalWrite(buzzer, LOW);
        delay(15);
      }

      User = "leer";
      delay(1000);
      lcd_bitte_anmelden();

      digitalWrite(SS_PIN_522, HIGH);
      digitalWrite(SS_PIN_LS, HIGH);

      return;
    }
  }



  lcd.setCursor(15, 0);
  lcd.print("   ");
  a = 0;

  //Schreiben der Daten in feuerwehr.tempanmeldung (Sperrdatei)

  sprintf(SQL_0, INSERT_TEXT_TEMP, Mitgliedsnummer_c);

  my_conn.cmd_query(SQL_0);

  SQL_0[0] = (char)0;

  //Schreiben der Daten in feuerwehr.logging (Erfassungsdatei)
  User = "'" + User + "'";

  int User_len = User.length() + 1;
  char User_c[User_len + 1];
  User.toCharArray(User_c, User_len);

  sprintf(SQL_0, INSERT_TEXT_LOG, Mitgliedsnummer_c, User_c);

  my_conn.cmd_query(SQL_0);

  SQL_0[0] = (char)0;



  digitalWrite(SS_PIN_522, HIGH);
  digitalWrite(SS_PIN_LS, HIGH);

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Akzeptiert");
  User = "leer";
  delay(1000);
  lcd_bitte_anmelden();

  return ;
}


//Funktion für Rahmendaten auf dem Displays
void lcdgrundansicht()
{ lcd.clear();
  lcd.print("Mitglied:");
  //lcd.setCursor(0, 2);
  //lcd.print("Log:");
  lcd.setCursor(0, 3);
  lcd.print("Karte:");
}

//
void lcd_schon_angemeldet()
{ lcd.clear();
  lcd.print("Schon angemeldet");

}


//Funktion für Grundansicht auf dem Display
void lcd_bitte_anmelden()
{ lcd.clear();
  lcd.print("Jugendfeuerwehr");

  lcd.setCursor(0, 1);
  lcd.print("Musterstadt");
  lcd.setCursor(0, 3);
  lcd.print("Bitte anmelden");
  Wappen();
}
void Wappen() {
  lcd.setCursor(17, 1);
  lcd.write(0);
  lcd.setCursor(18, 1);
  lcd.write(1);
  lcd.setCursor(19, 1);
  lcd.write(2);
  lcd.setCursor(17, 2);
  lcd.write(3);
  lcd.setCursor(18, 2);
  lcd.write(4);
  lcd.setCursor(19, 2);
  lcd.write(5);
}



void karte_ungueltig()
{ lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Fehler: ");
  lcd.setCursor(0, 1);
  lcd.print("Karte ung\365ltig.");
  delay(1000);
  lcd_bitte_anmelden();
}


void inBearbeitung() {

  lcd.clear();
  lcd.setCursor(0, 1);
  lcd.print("...in Bearbeitung");
  //delay (100);

}

//SQL-Funktionen
void do_query(const char *q) {

  column_names *c; // pointer to column values
  row_values *r;   // pointer to row values

  // First, execute query. If it returns a value pointer,
  // we have a result set to process. If not, we exit.
  if (!my_conn.cmd_query(q)) {
    return;
  }

  // Next, we read the column names and display them.
  //
  // NOTICE: You must *always* read the column names even if
  //         you do not use them. This is so the connector can
  //         read the data out of the buffer. Row data follows the
  //         column data and thus must be read first.
  c = my_conn.get_columns();
  for (int i = 0; i < c->num_fields; i++) {
    Serial.print(c->fields[i]->name);
    if (i < c->num_fields - 1) {
      Serial.print(",");
    }
  }
  Serial.println();

  // Next, we use the get_next_row() iterator and read rows printing
  // the values returned until the get_next_row() returns NULL.
  int num_cols = c->num_fields;
  rows = 0;
  do {
    r = my_conn.get_next_row();
    if (r) {
      rows++;
      for (int i = 0; i < num_cols; i++) {
        Serial.print(r->values[i]);
        value_return = String(r->values[i]);

        Serial.print(value_return);

        if (i < num_cols - 1) {
          Serial.print(", ");
        }
      }
      Serial.println();
      // Note: we free the row read to free the memory allocated for it.
      // You should do this after you've processed the row.
      my_conn.free_row_buffer();
    }
  } while (r);
  Serial.print(rows);
  Serial.println(" rows in result.");

  // Finally, we are done so we free the column buffers
  my_conn.free_columns_buffer();
}

void Kartenlesen () {
  for (int i = 0; i < mfrc522.uid.size; i++) {
    readCard[i] = mfrc522.uid.uidByte[i];

  }

  Cardteil0 = String (readCard[0], HEX);
  if (Cardteil0.length() < 2) {
    Cardteil0 = "0" + Cardteil0;
  }
  Cardteil1 = String (readCard[1], HEX);
  if (Cardteil1.length() < 2) {
    Cardteil1 = "0" + Cardteil1;
  }
  Cardteil2 = String (readCard[2], HEX);
  if (Cardteil2.length() < 2) {
    Cardteil2 = "0" + Cardteil2;
  }
  Cardteil3 = String (readCard[3], HEX);
  if (Cardteil3.length() < 2) {
    Cardteil3 = "0" + Cardteil3;
  }
}

